package com.xxl.job.executor.jobhandler;

import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;
import com.xxl.job.executor.common.entity.CtopOauthToken;
import com.xxl.job.executor.common.service.ICtopOauthTokenService;
import com.xxl.job.executor.report.service.IByteDanceVideoReportDailyService;
import com.xxl.job.executor.report.service.IBytedanceReportService;
import com.xxl.job.executor.utils.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

@Component
public class BytedanceDailyMaterialReportJob {
    @Autowired
    private ICtopOauthTokenService tokenService;
    @Autowired
    private IBytedanceReportService bytedanceReportService;

    @XxlJob("bytedanceDailyMaterialReportJob")
    public void execute() {
        String param = XxlJobHelper.getJobParam();
        if (param==null || param.trim().length()==0) {
            XxlJobHelper.log("param["+ param +"] invalid.");
            XxlJobHelper.handleFail();
            return;
        }
        XxlJobHelper.log("头条广告组时报获取任务执行开始(账户id:{})",param);
        CtopOauthToken token = tokenService.getOauthTokenByAccountId(param);
        if (null == token) {
            XxlJobHelper.log("获取头条token数据异常：账户{}token不存在",param);
            XxlJobHelper.handleFail();
            return;
        }
        Long start = System.currentTimeMillis();
        Date getDate = DateUtils.addDay(new Date(), -1);
        String date = DateUtils.formatDate(getDate);
        XxlJobHelper.log("头条获取素材报表数据任务开始，任务时间："+ date);
        try {
            //获取头条素材报表两天前的数据
            bytedanceReportService.bytedanceMaterialReport(token, date, date);
        } catch (Exception e) {
            e.printStackTrace();
        }
        Long end = System.currentTimeMillis();
        XxlJobHelper.log("头条素材数据获取耗时：{}毫秒", end - start);
    }
}
